/*
 * Segment 2 (KEx),  SWE 2.DE Statischer Import MIF
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.mif.konfigdaten;

import java.util.ArrayList;

import org.geotools.feature.FeatureType;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Geometry;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.kex.mif.kb.KonfigurationsObjekt;
import de.bsvrz.sys.funclib.bitctrl.geolib.WGS84Polygon;
import de.bsvrz.sys.funclib.bitctrl.geolib.WGS84Punkt;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Abstraktes Konfigurationsobjekt.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */
public abstract class AbstraktKonfigurationsObjekt implements KonfigurationsObjekt {

	/**
	 * Koordinaten.
	 */
	protected ArrayList<WGS84Punkt> wgs84 = new ArrayList<WGS84Punkt>();

	/**
	 * L&auml;nge.
	 */
	protected double laenge;

	/**
	 * PID.
	 */
	protected String pid;

	/**
	 * Name.
	 */
	protected String name;

	/**
	 * Typ im Datenkatalog.
	 */
	SystemObjectType davtype;

	/**
	 * Name des Typs im Datenkatalog.
	 */
	protected String davobjektyp;

	/**
	 * Flag, ob das Objekt importiert werden soll.
	 */
	private boolean importieren = true;

	/**
	 * der zugeordnete Feature-Typ.
	 */
	private final FeatureType featuretyp;

	/**
	 * die Geometrie.
	 */
	protected Geometry geometrie;

	/**
	 * DebugLogger für Debug-Ausgaben.
	 */
	private static final Debug DEBUG = Debug.getLogger();

	/**
	 * Konstruktor.
	 *
	 * @param pid
	 *            PID
	 * @param name
	 *            Name
	 * @param davobjekttyp
	 *            Zieltyp im DAV
	 * @param geometrie
	 *            Geometrie
	 * @param featuretyp
	 *            FeatureType
	 */
	public AbstraktKonfigurationsObjekt(final String pid, final String name, final String davobjekttyp,
			final Geometry geometrie, final FeatureType featuretyp) {
		this.pid = pid;
		this.name = name;
		this.davobjektyp = davobjekttyp;
		this.geometrie = geometrie;
		this.featuretyp = featuretyp;
		if (davobjekttyp.equals(KonfigurationsDaten.DAVTYP_UNBESTIMMT)) {
			importieren = false;
		}
	}

	@Override
	public void addKoordinaten(final ArrayList<WGS84Punkt> punkte) {
		this.wgs84.addAll(punkte);
		final WGS84Polygon p = new WGS84Polygon(wgs84);
		laenge = p.laenge();
	}

	/**
	 * Erzeugt ein {@link Data} Objekt.
	 *
	 * @param dav
	 *            {@link ClientDavConnection}
	 * @param atg
	 *            Attributgruppe
	 * @param asp
	 *            Aspekt
	 * @return {@link Data}
	 */
	public Data createData(final ClientDavConnection dav, final String atg, final String asp) {
		final AttributeGroup attributeGroup = dav.getDataModel().getAttributeGroup(atg);
		// Aspect aspect = dav.getDataModel().getAspect(asp);
		return dav.createData(attributeGroup);
	}

	@Override
	public String getDavobjektyp() {
		return davobjektyp;
	}

	@Override
	public SystemObjectType getDavTyp() {
		return davtype;
	}

	/**
	 * Gibt den FeatureType des assoziierten Features zur&uuml;ck.
	 *
	 * @return FeatureType
	 */
	public FeatureType getFeaturetyp() {
		return featuretyp;
	}

	@Override
	public Geometry getGeometrie() {
		return geometrie;
	}

	@Override
	public ArrayList<WGS84Punkt> getKoordinaten() {
		return this.wgs84;
	}

	@Override
	public double getLaenge() {
		return laenge;
	}

	@Override
	public String getName() {
		return name;
	}

	@Override
	public String getPid() {
		return pid;
	}

	@Override
	public void setImportieren(final boolean importieren) {
		this.importieren = importieren;
	}

	@Override
	public void setKoordinaten(final ArrayList<WGS84Punkt> punkte) {
		this.wgs84 = punkte;
		final WGS84Polygon p = new WGS84Polygon(wgs84);
		laenge = p.laenge();
	}

	@Override
	public void setKoordinaten(final Coordinate[] featureKoordinatenWGS84) {
		for (final Coordinate element : featureKoordinatenWGS84) {
			// WGS84Koordinate w = GeoTransformation.UTM_nach_WGS84Punkt(
			// new UTMKoordinate(featureKoordinatenWGS84[i].x,
			// featureKoordinatenWGS84[i].y, 32));
			AbstraktKonfigurationsObjekt.DEBUG
					.finest("setze Koordinaten '" + getPid() + "': " + element.x + ", " + element.y);
			this.wgs84.add(new WGS84Punkt(element.x, element.y));
		}
		final WGS84Polygon p = new WGS84Polygon(wgs84);
		laenge = p.laenge();
	}

	@Override
	public void setLaenge(final double laenge) {
		this.laenge = laenge;
	}

	@Override
	public void setName(final String name) {
		this.name = name;
	}

	@Override
	public void setPid(final String pid) {
		this.pid = pid;
	}

	@Override
	public void setZieltyp(final String zieltyp) {
		davobjektyp = zieltyp;
	}

	@Override
	public boolean sollImportieren() {
		return importieren;
	}
}
