/*
 * Segment 2 (KEx),  SWE 2.DE Statischer Import MIF
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */


package de.bsvrz.kex.mif.mif;

import org.geotools.factory.FactoryRegistryException;
import org.geotools.feature.Feature;
import org.geotools.feature.FeatureType;
import org.geotools.geometry.jts.JTS;
import org.geotools.referencing.CRS;
import org.geotools.referencing.crs.DefaultGeographicCRS;
import org.opengis.referencing.FactoryException;
import org.opengis.referencing.crs.CoordinateReferenceSystem;
import org.opengis.referencing.operation.MathTransform;
import org.opengis.referencing.operation.TransformException;
import org.opengis.spatialschema.geometry.MismatchedDimensionException;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Geometry;

import de.bsvrz.kex.mif.konfigdaten.ImportException;
import de.bsvrz.sys.funclib.bitctrl.geolib.WGS84Polygon;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * .
 * @author BitCtrl Systems GmbH, Gieseler
 */
public class MifObjekt {

	/** Feature. */
	private final Feature feature;

	/**
	 * DebugLogger für Debug-Ausgaben.
	 */
	private static final Debug DEBUG = Debug.getLogger();

	/**
	 * Erzeugt ein neues Objekt aus einem Feature.
	 *
	 * @param feature
	 *            assoziiertes Feature.
	 */
	public MifObjekt(final Feature feature) {
		this.feature = feature;
	}

	/**
	 * Gibt den Typ des Objektes zur&uuml;ck.
	 *
	 * @return {@link MifTyp}
	 *
	 * @throws ImportException bei Fehlern
	 */
	public MifTyp getMifTyp() throws ImportException {
		return MifTyp.bestimmeTyp(feature);
	}

	/**
	 * Gibt die Koordinatenliste als WGS84-Polygon zur&uuml;ck.
	 *
	 * @param featuretype
	 *            FeatureType
	 * @param geometrie
	 *            Geometrie
	 *
	 * @return {@link WGS84Polygon}
	 */
	public static WGS84Polygon getFeatureKoordinatenAlsWGS84Polygon(final FeatureType featuretype,
			final Geometry geometrie) {
		final Coordinate[] c = MifObjekt.getFeatureKoordinatenWGS84(featuretype, geometrie);
		final double[] x = new double[c.length];
		final double[] y = new double[c.length];

		for (int i = 0; i < c.length; i++) {
			x[i] = c[i].x;
			y[i] = c[i].y;
		}

		return new WGS84Polygon(x, y);
	}

	/**
	 * Gibt die Koordinatenliste eines Features als WGS84-Koordninaten
	 * zur&uuml;ck.
	 *
	 * @param featuretype
	 *            FeatureType
	 * @param geometrie
	 *            Geometrie
	 * @return Array von Koordinaten
	 *
	 *         <p>
	 *         Problem: wechselnde Transformationsklassen:
	 *         </p>
	 *         <p>
	 *         SIEHE:
	 *         http://www.mail-archive.com/geotools-gt2-users@lists.sourceforge.
	 *         net/msg05939.html<br>
	 *         This is well, known behaviour. For now, the rule is <br>
	 *         ONLY ONE EPSG JAR ON THE CLASSPATH<br>
	 *         Martin plans to fix this someday but there's a bunch of other
	 *         work on his stack.
	 *         </p>
	 */
	public static Coordinate[] getFeatureKoordinatenWGS84(final FeatureType featuretype, final Geometry geometrie) {
		// FeatureType ft = f.getFeatureType();
		// GeometryFactory gf = f.getDefaultGeometry().getFactory();
		MifObjekt.DEBUG.finest("getFeatureKoordinatenWGS84:featuretype: " + featuretype);
		MifObjekt.DEBUG.finest("getFeatureKoordinatenWGS84:getDefaultGeometry(): " + featuretype.getDefaultGeometry());
		MifObjekt.DEBUG.finest("getFeatureKoordinatenWGS84:getDefaultGeometry().getCoordinateSystem(): "
				+ featuretype.getDefaultGeometry().getCoordinateSystem());
		CoordinateReferenceSystem sourceCrs = featuretype.getDefaultGeometry().getCoordinateSystem();
		final CoordinateReferenceSystem destCrs = DefaultGeographicCRS.WGS84;

		if (sourceCrs == null) { // MIF!
			MifObjekt.DEBUG.finest("Source-Koordinatensystem wird bestimmt");
			try {
				// sourceCrs = CRS.decode("EPSG:32632"); // WGS84/UTM zone 32N
				// //$NON-NLS-1$

				// CoordSys Earth Projection 8, 1000, "m", 9, 0, 1, 3500000, 0
				// Bounds (-4748143.32561, -10000855.7646) (11748143.3256,
				// 10000855.7646)
				sourceCrs = CRS.decode("EPSG:31467"); //$NON-NLS-1$
			} catch (final FactoryRegistryException e1) {
				e1.printStackTrace();
			} catch (final FactoryException e1) {
				e1.printStackTrace();
			}
		}

		if ((sourceCrs != null) && !sourceCrs.equals(destCrs)) {
			MifObjekt.DEBUG.finest("Koordinaten werden transformiert");

			MifObjekt.DEBUG.finest("sourceCrs: " + sourceCrs);
			MifObjekt.DEBUG.finest("sourceCrs: " + sourceCrs.getClass().getName());
			MifObjekt.DEBUG.finest("sourceCrs: " + sourceCrs.getClass().getPackage().getName());
			MifObjekt.DEBUG.finest("destCrs: " + destCrs);

			final MathTransform trans;

			try {
				trans = CRS.findMathTransform(sourceCrs, DefaultGeographicCRS.WGS84, true);
				final Geometry newGeom = JTS.transform(geometrie, trans);
				MifObjekt.DEBUG.finest("trans: " + trans + ", org: " + geometrie.getCoordinates()[0] + ", new: "
						+ newGeom.getCoordinates()[0]);

				return newGeom.getCoordinates();
			} catch (final FactoryException e) {
				e.printStackTrace();
			} catch (final MismatchedDimensionException e) {
				e.printStackTrace();
			} catch (final TransformException e) {
				e.printStackTrace();
			}
		} else {
			return geometrie.getCoordinates();
		}

		return null;
	}

	/**
	 * Gibt das assoziierte Feature zur&uuml;ck.
	 *
	 * @return Feature
	 */
	public Feature getFeature() {
		return feature;
	}

	/**
	 * Gibt die Geometrie des Objektes zur&uuml;ck.
	 *
	 * @return Geometrie
	 */
	public Geometry getGeometrie() {
		return feature.getDefaultGeometry();
	}

	/**
	 * Testet, ob es sich bei dem Objekt um ein Text-Objekt handelt.
	 *
	 * @return true, wenn Text-Objekt, sonst false
	 */
	public boolean istTextObjekt() {
		try {
			if (getMifTyp() == MifTyp.MIF_PUNKT) {
				final String text = getText();
				return (text != null) && (text.length() > 0);
			}
		} catch (final ImportException e) {
			return false;
		}

		return false;
	}

	/**
	 * Gibt den Textwert eines Text-Objektes zur&uuml;ck.
	 *
	 * @return Text
	 */
	public String getText() {
		// return (String) _feature.getAttribute(MifTyp.MIF_TEXT_ATTRIBUT_NAME);
		String text = (String) feature.getAttribute(MifTyp.MIF_TEXT_ATTRIBUT_NAME);
		text = text.replaceAll("-\\\\n", "-").replaceAll("\\\\n", " ");
		final int indexOf = text.indexOf("-");
		if (indexOf != -1) {
			if (Character.isLowerCase(text.charAt(indexOf + 1))) {
				text = text.replaceAll("-", "");
			}
		}
		return text;
	}
}
